# Generated by Django 4.0.4 on 2022-11-29 21:40

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("j2fa", "0003_auto_20180807_2202"),
    ]

    operations = [
        migrations.AddField(
            model_name="twofactorsession",
            name="email",
            field=models.EmailField(blank=True, default="", max_length=254),
        ),
    ]
