# -*- coding: utf-8 -*-
# Generated by Django 1.11.7 on 2017-11-02 01:58
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import django.utils.timezone


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name="TwoFactorSession",
            fields=[
                ("id", models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID")),
                ("created", models.DateTimeField(blank=True, db_index=True, default=django.utils.timezone.now)),
                ("user_agent", models.CharField(max_length=512)),
                ("ip", models.GenericIPAddressField(db_index=True)),
                ("phone", models.CharField(db_index=True, max_length=32)),
                ("code", models.CharField(max_length=8)),
                ("active", models.BooleanField(db_index=True, default=False)),
                ("archived", models.BooleanField(db_index=True, default=False)),
                (
                    "user",
                    models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name="+", to=settings.AUTH_USER_MODEL),
                ),
            ],
        ),
    ]
